import type { AxiosResponse } from 'axios';
import type { Readable } from 'stream';
import type { Bucket, ConfigSchemaCredentials, ListPage, MetadataResponseHeaders } from './types';
import type { BinaryData } from '../BinaryData/types';
export declare class ObjectStoreService {
    private host;
    private bucket;
    private credentials;
    private isReady;
    private isReadOnly;
    private logger;
    init(host: string, bucket: Bucket, credentials: ConfigSchemaCredentials): Promise<void>;
    setReadonly(newState: boolean): void;
    setReady(newState: boolean): void;
    checkConnection(): Promise<AxiosResponse<unknown, any> | undefined>;
    put(filename: string, buffer: Buffer, metadata?: BinaryData.PreWriteMetadata): Promise<AxiosResponse<any, any>>;
    get(fileId: string, { mode }: {
        mode: 'buffer';
    }): Promise<Buffer>;
    get(fileId: string, { mode }: {
        mode: 'stream';
    }): Promise<Readable>;
    getMetadata(fileId: string): Promise<MetadataResponseHeaders>;
    deleteOne(fileId: string): Promise<AxiosResponse<unknown, any>>;
    deleteMany(prefix: string): Promise<AxiosResponse<unknown, any> | undefined>;
    list(prefix: string): Promise<{
        key: string;
        lastModified: string;
        eTag: string;
        size: number;
        storageClass: string;
    }[]>;
    getListPage(prefix: string, nextPageToken?: string): Promise<ListPage>;
    private toPath;
    private blockWrite;
    private request;
}
