"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.LoadOptionsContext = void 0;
const lodash_1 = require("lodash");
const ExtractValue_1 = require("../ExtractValue");
const NodeExecuteFunctions_1 = require("../NodeExecuteFunctions");
const node_execution_context_1 = require("./node-execution-context");
class LoadOptionsContext extends node_execution_context_1.NodeExecutionContext {
    constructor(workflow, node, additionalData, path) {
        super(workflow, node, additionalData, 'internal');
        this.path = path;
        this.helpers = {
            ...(0, NodeExecuteFunctions_1.getSSHTunnelFunctions)(),
            ...(0, NodeExecuteFunctions_1.getRequestHelperFunctions)(workflow, node, additionalData),
        };
    }
    async getCredentials(type) {
        return await (0, NodeExecuteFunctions_1.getCredentials)(this.workflow, this.node, type, this.additionalData, this.mode);
    }
    getCurrentNodeParameter(parameterPath, options) {
        const nodeParameters = this.additionalData.currentNodeParameters;
        if (parameterPath.charAt(0) === '&') {
            parameterPath = `${this.path.split('.').slice(1, -1).join('.')}.${parameterPath.slice(1)}`;
        }
        let returnData = (0, lodash_1.get)(nodeParameters, parameterPath);
        if (options?.extractValue) {
            const nodeType = this.workflow.nodeTypes.getByNameAndVersion(this.node.type, this.node.typeVersion);
            returnData = (0, ExtractValue_1.extractValue)(returnData, parameterPath, this.node, nodeType);
        }
        return returnData;
    }
    getCurrentNodeParameters() {
        return this.additionalData.currentNodeParameters;
    }
    getNodeParameter(parameterName, fallbackValue, options) {
        const runExecutionData = null;
        const itemIndex = 0;
        const runIndex = 0;
        const connectionInputData = [];
        return (0, NodeExecuteFunctions_1.getNodeParameter)(this.workflow, runExecutionData, runIndex, connectionInputData, this.node, parameterName, itemIndex, this.mode, (0, NodeExecuteFunctions_1.getAdditionalKeys)(this.additionalData, this.mode, runExecutionData), undefined, fallbackValue, options);
    }
}
exports.LoadOptionsContext = LoadOptionsContext;
//# sourceMappingURL=load-options-context.js.map