import type { Request, Response } from 'express';
import type { CloseFunction, ICredentialDataDecryptedObject, IDataObject, IGetNodeParameterOptions, INode, IRunExecutionData, IWebhookData, IWebhookFunctions, IWorkflowExecuteAdditionalData, NodeConnectionType, NodeParameterValueType, WebhookType, Workflow, WorkflowExecuteMode } from 'n8n-workflow';
import { NodeExecutionContext } from './node-execution-context';
export declare class WebhookContext extends NodeExecutionContext implements IWebhookFunctions {
    private readonly webhookData;
    private readonly closeFunctions;
    private readonly runExecutionData;
    readonly helpers: IWebhookFunctions['helpers'];
    readonly nodeHelpers: IWebhookFunctions['nodeHelpers'];
    constructor(workflow: Workflow, node: INode, additionalData: IWorkflowExecuteAdditionalData, mode: WorkflowExecuteMode, webhookData: IWebhookData, closeFunctions: CloseFunction[], runExecutionData: IRunExecutionData | null);
    getCredentials<T extends object = ICredentialDataDecryptedObject>(type: string): Promise<T>;
    getBodyData(): IDataObject;
    getHeaderData(): import("http").IncomingHttpHeaders;
    getParamsData(): object;
    getQueryData(): object;
    getRequestObject(): Request;
    getResponseObject(): Response;
    private assertHttpRequest;
    getNodeWebhookUrl(name: WebhookType): string | undefined;
    getWebhookName(): WebhookType;
    getInputConnectionData(inputName: NodeConnectionType, itemIndex: number): Promise<unknown>;
    evaluateExpression(expression: string, evaluateItemIndex?: number): import("n8n-workflow").NodeParameterValue | import("n8n-workflow").INodeParameters | import("n8n-workflow").INodeParameters[] | import("n8n-workflow").NodeParameterValue[];
    getNodeParameter(parameterName: string, fallbackValue?: any, options?: IGetNodeParameterOptions): NodeParameterValueType | object;
}
